"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var path = require("path");
var Git_1 = require("./utils/Git");
var YmlLoader_1 = require("./YmlLoader");
function loadRepo(dir) {
    var rootYml = rootRepo(dir);
    var dependencies = YmlLoader_1.loadYml(dir);
    rootYml.children = loadChildren(dir, dependencies);
    return rootYml;
}
exports.loadRepo = loadRepo;
function rootRepo(dir) {
    var name = path.parse(dir).name;
    return {
        name: name,
        dir: dir,
        repoPath: ".",
        repoURL: Git_1.readRemote(dir),
        parent: "root",
        children: [],
    };
}
function loadChildren(dir, deps) {
    var children = deps.map(function (d) {
        var ymlDir = path.resolve(dir, d.repoPath);
        var dependencies = YmlLoader_1.loadYml(ymlDir);
        var nestedChildren = loadChildren(ymlDir, dependencies);
        return __assign({}, d, { dir: ymlDir, parent: dir, children: nestedChildren });
    });
    return children;
}
function loadFlattedDep(dir) {
    var res = loadRepo(dir);
    return flatDependencies(res);
}
exports.loadFlattedDep = loadFlattedDep;
/**
 * 树状结构展开为一维数组。
 * 当 dir 字段内容相同时，上层覆盖下层；同一层，右边覆盖左边的内容。
 * 返回的内容，从顶端左上角返回
 * @param yml
 */
function flatDependencies(yml) {
    var nestedSub = yml.children.map(function (sub) {
        return flatDependencies(sub);
    });
    return deduplicate(yml.children.concat(flatTwoDimensionalArray(nestedSub, "dir")), "dir");
}
exports.flatDependencies = flatDependencies;
/**
 * 将二维数组转化为一维数组，同时根据 key 字段进行去重处理。
 * @param two 二维数组
 * @param key 去重关键字
 * @param reverse 选择重复时，保留的元素。true 时，保留后面的元素；false 时，保留前面元素。默认为 false，保留前面的元素。（与 deduplicate 一致）
 */
function flatTwoDimensionalArray(two, key, reverse) {
    if (reverse === void 0) { reverse = true; }
    return two.reduce(function (p, c) {
        return reverse ? deduplicate(p.concat(c).reverse(), key) : deduplicate(p.concat(c), key);
    }, []);
}
/**
 * 根据 key ，对 list 进行去重处理。如果 key 字段相同，会保留前面。
 * @param list 数组
 * @param key 去重关键字
 */
function deduplicate(list, key) {
    var keys = new Set();
    return list.filter(function (v) {
        var contains = keys.has(v[key]);
        keys.add(v[key]);
        return !contains;
    });
}
//# sourceMappingURL=RepoLoader.js.map