"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs");
var path = require("path");
var Git_1 = require("./utils/Git");
var FileHandler_1 = require("./utils/FileHandler");
var yaml = require("js-yaml");
var resourceContent = {
    name: 'string',
    repoPath: 'string',
    repoURL: 'string',
};
var lockContent = {
    name: 'string',
    commit: 'string',
};
var carrot_yml = "carrot.yml";
var lock_yml = "lock.yml";
function findYmlDir(dir) {
    if (dir === void 0) { dir = FileHandler_1.WorkDir; }
    if (FileHandler_1.getPathState(path.resolve(dir, carrot_yml)) === FileHandler_1.FileState.File) {
        console.log("find yml in: " + FileHandler_1.relativePath(dir));
        return dir;
    }
    if (dir === "/") {
        throw new Error("no " + carrot_yml + " exist or any of the parent directories");
    }
    var parentDir = path.resolve(dir, "..");
    return findYmlDir(parentDir);
}
exports.findYmlDir = findYmlDir;
function writeYml(dir, data) {
    if (!data || data.length == 0) {
        return;
    }
    dir = parseLegacyYmlPath(dir);
    fs.writeFileSync(path.resolve(dir, carrot_yml), yaml.dump(data));
    // TODO: 资源与 commit 锁分开存储
}
exports.writeYml = writeYml;
function loadYml(dir) {
    dir = parseLegacyYmlPath(dir);
    var carrots = loadCarrot(dir);
    var locks = loadLock(dir);
    return mergeConfig(carrots, locks);
}
exports.loadYml = loadYml;
function initYml(dir) {
    if (dir === void 0) { dir = FileHandler_1.WorkDir; }
    var data = loadYml(dir);
    for (var _i = 0, data_1 = data; _i < data_1.length; _i++) {
        var subConfig = data_1[_i];
        var commit = Git_1.readCommit(path.resolve(dir, subConfig.repoPath));
        var repoURL = Git_1.readRemote(path.resolve(dir, subConfig.repoPath));
        subConfig.commit = commit;
        subConfig.repoURL = repoURL;
    }
    writeYml(FileHandler_1.WorkDir, data);
}
exports.initYml = initYml;
function parseLegacyYmlPath(dir) {
    if (path.basename(dir) === carrot_yml) {
        dir = path.dirname(dir);
    }
    return dir;
}
function loadCarrot(dir) {
    var carrotPath = path.resolve(dir, carrot_yml);
    if (FileHandler_1.getPathState(carrotPath) === FileHandler_1.FileState.File) {
        return yaml.safeLoad(fs.readFileSync(carrotPath, 'utf-8'));
    }
    else {
        return [];
    }
}
function loadLock(dir) {
    var lockPath = path.resolve(dir, lock_yml);
    if (FileHandler_1.getPathState(lockPath) === FileHandler_1.FileState.File) {
        return yaml.safeLoad(fs.readFileSync(lockPath, 'utf-8'));
    }
    else {
        return [];
    }
}
function mergeConfig(resources, locks) {
    return resources.map(function (r) { return Object.assign(r, locks.find(function (l) { return l.name === r.name; })); });
}
function splitConfig(module) {
    var resource = {};
    var lock = {};
    for (var key in module) {
        if (module.hasOwnProperty(key)) {
            var value = module[key];
            if (keyInType(key, resourceContent)) {
                resource[key] = value;
            }
            if (keyInType(key, lockContent)) {
                lock[key] = value;
            }
        }
    }
    return { resource: resource, lock: lock };
}
// TODO: 强约束 x:string 。但是这样要增加对 T 的约束，保证 keyof T 出来的是 string，暂时不会写
function keyInType(x, instance) {
    return x in instance;
}
//# sourceMappingURL=YmlLoader.js.map