#!/usr/bin/env node
const rabbit = require("./dist");

require("yargs")
.command("load", "打印树状依赖", {}, (argv) => {
    rabbit.load(argv);
})
.command("flat", "打印平铺后的依赖", {}, argv => {
    rabbit.flatDep(argv);
})
.command("$0", "等于 update", {}, (argv) => {
    rabbit.update(argv);
})
.command("checkout", "将依赖库 checkout 到对应 commit", {}, (argv) => {
    rabbit.checkout(argv);
})
.command("update", "从底层依赖开始向上更新 commit", {
    "s": {
        alias: "specify",
        describe: "指定升级yaml 中单个 repo"
    }
}, (argv) => {
    rabbit.update(argv);
})
.command("pull", "更新所有依赖库", {}, (argv) => {
    rabbit.pull(argv);
})
.command("check", "校验 yml，不一致时中断检查", {}, (argv) => {
    rabbit.check(argv);
})
.command("state", "格式化输出 commit 匹配结果，以红色显示不匹配的部分", {}, (argv) => {
    rabbit.state(argv);
})
.command("stash", "stash 所有未提交改动", {}, argv => {
    rabbit.stash(argv);
})
.command("reset", "强制 stash 所有未提交的改动，然后根据 yml commit 回滚依赖库", {}, (argv) => {
    rabbit.clone({...argv, hard: true})
})
.command("clone", "递归 clone 所有 yaml 中的依赖链", {
    "hard": {
        describe: "强制 stash 所有未提交的改动，然后进行 clone reset 操作"
    }
}, function(argv) {
    rabbit.clone(argv);
})
.command("clean", "文件级别清理，不会修改文件本身。", {
    "d": {
        describe: "递归删除目录内容。当 clean API 没有传入 <path> 参数时，未追踪目录中的文件，不会被删除。传入 -d 选项时，会执行递归删除。",
    },
    "force": {
        alias: "f",
        describe: "强制删除。git config 中 clean.requireForce 为 false 时，git 不会删除文件。必须传入 -f 或者使用 -i 交互式操作，git clean 才会删除文件"
    },
    "interactive": {
        alias: "i",
        describe: "交互式，暂时不支持"
    },
    "dry-run": {
        alias: "n",
        describe: "dry run，只显示要做什么，不实际执行"
    },
    "quiet": {
        alias: "q",
        describe: "安静模式，只反馈报错"
    },
    "exclude": {
        command: "exclude=<pattern>",
        alias: "e",
        describe: "排除。<pattern> 添加额外的排除规则，保证该表达式命中的文件，不被删除",
    },
    "x": {
        describe: "删除所有未追踪的文件。不跟随 gitignore 规则，但接受 -e 参数。gitignore 中的文件，如果从未追踪过，就会被删除。"
    },
    "X": {
        describe: "仅删除 gitignore 中的文件"
    }
}, function(argv) {
    rabbit.clean(argv);
})
.command("test", "测试用", {
}, function(argv) {
    console.log(argv);
})
.options({
    'verbose': {
        alias: 'v',
        describe: "输出详细信息"
    }
})
.argv;