import * as path from "path";
import * as fs from "fs";
import * as child_process from "child_process";

export const WorkDir = child_process.execSync(`pwd`).toString().trim();

export enum FileState {
    None = "none",
    Dir = "directory",
    File = "file"
}

export function relativePath(dir: string): string {
    return path.normalize(path.relative(WorkDir, dir));
}

/**
 * 不再检测 SymbolicLink，直接根据 path 读取最终内容；
 * 如果需要识别 SymbolicLink，应该使用 fs.lstatSync;
 * 当不存在内容时，statSync 会直接 throw error
 */ 
export function getPathState(dir: string): FileState {
    try {
        const stats = fs.statSync(dir);
        if (stats.isDirectory()) {
            return FileState.Dir;
        } else if (stats.isFile()) {
            return FileState.File;
        } else {
            return FileState.None;
        }
    } catch (error) {
        return FileState.None;
    }
}